
//+------------------------------------------------------------------+
//|                                          LaguerreFilterCloud.mq5 |
//| Laguerre Filter                                                  |
//|                                                                  |
//| Algorithm taken from book                                        |
//|     "Cybernetics Analysis for Stock and Futures"                 |
//| by John F. Ehlers                                                |
//|                                                                  |
//|                                              contact@mqlsoft.com |
//|                                          http://www.mqlsoft.com/ |
//|   modify to MQL4 by Mikhail Kozhemyako > LaguerreFilterCloud.mq4 |
//+------------------------------------------------------------------+
#property copyright "Coded by Witold Wozniak"
#property link      "www.mqlsoft.com"
#property version   "1.00"
#property strict
#property indicator_separate_window 
#property indicator_buffers 1
#property indicator_type1   DRAW_HISTOGRAM
#property indicator_color1  clrGreen
#property indicator_width1 3
#property indicator_label1  "Laguerre Filter"
//+----------------------------------------------+
//|                    |
//+----------------------------------------------+
input double Gamma=0.68;//   
int oldbar;
double L0,L1,L2,L3,L0A,L1A,L2A,L3A;
int min_rates_total;
double LaguerreBuffer,FirBuffer,DifBuffer[];
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+  
void OnInit()
  {
//----     
   min_rates_total=4;
   oldbar=Bars-1;
//----      
   SetIndexBuffer(0,DifBuffer);
   SetIndexShift(0,min_rates_total+1);
//----      
   string shortname=StringConcatenate("Laguerre Filter(",DoubleToString(Gamma,3),")");
   IndicatorShortName(shortname);
  }
//+------------------------------------------------------------------+
//| Custom indicator iteration function                              |
//+------------------------------------------------------------------+
int OnCalculate(
                const int rates_total,    //       
                const int prev_calculated,//       
                const datetime &time[],
                const double &open[],
                const double& high[],     //       
                const double& low[],      //        
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[]
                )
  {
//----       
   if(rates_total<min_rates_total) return(0);
   int first,bar;
//----    first    
   if(prev_calculated>rates_total || prev_calculated<=0) //      
     {
      first=rates_total-1; //      
      bar=first;
     }
   else first=rates_total-prev_calculated+1+min_rates_total; //      
//----    
   for(bar=first;bar>=0 && !IsStopped(); bar--)
     {
      if(oldbar<Bars || prev_calculated==0)
         {
          oldbar=Bars;
          L0A = L0;
          L1A = L1;
          L2A = L2;
          L3A = L3;
         }
      L0 = (1 - Gamma) * (high[bar]+low[bar])/2 + Gamma * L0A;
      L1 = - Gamma * L0 + L0A + Gamma * L1A;
      L2 = - Gamma * L1 + L1A + Gamma * L2A;
      L3 = - Gamma * L2 + L2A + Gamma * L3A;
      if(bar>min_rates_total)
        {
         LaguerreBuffer=(L0+2.0*L1+2.0*L2+L3)/6.0;
         FirBuffer=(1.0*(high[bar]+low[bar])/2+2.0*(high[bar-1]+low[bar-1])/2+2.0*(high[bar-2]+low[bar-2])/2+1.0*(high[bar-3]+low[bar-3])/2)/6.0;
         DifBuffer[bar]=FirBuffer-LaguerreBuffer;
        }
      else
        {
         DifBuffer[bar]=0;
        }
     }
//----     
   return(rates_total);
  }
